//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
/* ------------------------ */
/* --- Global value         */
/* ------------------------ */

#define SLAVE_ADDRESS 0x50

#define VOICEINT  10
#define IDLETIME  6000 

/* --- I2C receive buffer */
unsigned char  rxBuffer[32];
/* --- audio output buffer */
unsigned char  outBuffer[32];
/* --- buffer pointer */
char outChunk;
/* --- rom address */
unsigned short dataAddress;
/* --- audio data size */
unsigned short dataLength;
/* --- audio output working flag */
char audioSw;
/* --- next data */
char requestI2C;
char requestSize;

/* --- timer count */
short  gIntCount;
short  gIdleCount;

/* --- puti count */
unsigned char  gPutiCount;

/* --- voice count */
unsigned char  gVoiceCount;

/* ------------------------ */
/* --- Timer interrupt      */
/* ------------------------ */
#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	unsigned char  l, i;

	/* --- output sw */
	if (audioSw	== 0) {
		DAC8_1_WriteStall(0x80);
		return;
	}

	/* --- DAC output */	
	l = outBuffer[outChunk];
	DAC8_1_WriteStall(l);

	outChunk++;
	if (outChunk == 32) {
		outChunk = 0;
		for(i=0; i<32; i++) {
			l = rxBuffer[i];
			if (l == 255) {
				l = 254;
			}
			outBuffer[i] = l;
		}
		/* - I2C */
		if (dataLength > 32) {
			requestSize = 32;
		}
		else {
			requestSize = dataLength;
		}
	    I2CHW_1_fReadBytes(SLAVE_ADDRESS, rxBuffer, requestSize, I2CHW_1_CompleteXfer ); 
		/* - next */
		dataAddress += requestSize;
		dataLength -= requestSize;
		if (dataLength == 0) {
			audioSw = 0;
		}
	}	
}


/* ------------------ */
/* --- wait timer */
#pragma interrupt_handler Timer2Int
void Timer2Int(void)
{
	if (gIntCount > 0) {
		gIntCount--;
	}
	if (gIdleCount < IDLETIME+1) {
		gIdleCount++;
	}
}

void WaitTimer(void)
{
	while(gIntCount > 0);
}



/* ------------------------ */
/* --- voice request        */
/* ------------------------ */
void AudioRequest(unsigned short address, unsigned short length)
{
	int  i;
	BYTE  addr[2];

	if (audioSw == 1) {
		return;
	}

	dataAddress = address;
	dataLength = length;
	addr[0] = (dataAddress >> 8) & 0xff;
	addr[1] = (dataAddress & 0xff);
	I2CHW_1_bWriteBytes(SLAVE_ADDRESS, addr, 2, I2CHW_1_CompleteXfer);
    /* Wait until the data is transferred */  
    while(!I2CHW_1_bReadI2CStatus() & I2CHW_WR_COMPLETE);  
    /* Clear Write Complete Status bit */  
    I2CHW_1_ClrWrStatus();  
	for(i=0; i<32; i++) {
		rxBuffer[i] = 0x80;
		outBuffer[i] = 0x80;
	}

	audioSw = 1;
}





/* ------------------------ */
/* --- Start                */
/* ------------------------ */
void main()
{
	BYTE  addr[2];
	char  i;

	/* --- buffer clear */
	for(i=0; i<32; i++) {
		rxBuffer[i] = 0x80;
		outBuffer[i] = 0x80;
	}
	audioSw = 0;
	outChunk = 0;

	/* --- module initialize */
	DAC8_1_Start(DAC8_1_LOWPOWER);
	LPF2_1_Start(LPF2_1_LOWPOWER);
	Counter8_1_Start();
	Counter8_2_Start();
	I2CHW_1_Start();
	I2CHW_1_EnableMstr();
	I2CHW_1_EnableInt();
	
	/* --- Start Interrupt */
	Counter8_1_EnableInt();
	Counter8_2_EnableInt();
	M8C_EnableGInt;
	
	dataAddress = 0;
	outChunk = 0;
	audioSw = 0;
	requestI2C = 0;
	
	gPutiCount = 0;
	gVoiceCount = 0;
	gIntCount = 0;
	gIdleCount = 0;

	while(1) {
		/* --- button */
		while((PRT0DR & 0x14) == 0) {
			if (gIdleCount == IDLETIME) {
				gIdleCount++;
				AudioRequest(35244, 9136);
			}
		}
		gIdleCount = 0;
		/* - puti */
		gPutiCount++;
		if (gPutiCount == VOICEINT) {
			gPutiCount = 0;
			switch(gVoiceCount) {
			case 0:
				AudioRequest(2376, 8448);
				break;
			case 1:
				AudioRequest(10824, 6144);
				break;
			case 2:
				AudioRequest(16968, 7640);
				break;
			case 3:
				AudioRequest(24608, 10636);
				break;
			}
			gVoiceCount++;
			if (gVoiceCount == 4) {
				gVoiceCount = 0;
			}
		}
		else {
			/* - normal puti */
			AudioRequest(0, 2376);			
		}
		/* dont chat*/
		gIntCount = 16;
		WaitTimer();
		/* release wait */
		while((PRT0DR & 0x14) != 0);

	}
}
